'use strict';

const fs = require('fs');
const path = require('path');

const axios = require('axios');

const externalFilePath = 'pagean-external-scripts';

const shouldSaveFile = (script, page) => {
    const scriptUrl = new URL(script);
    const pageUrl = new URL(page);
    return scriptUrl.host !== pageUrl.host;
};

const saveExternalScript = async (script) => {
    const result = { url: script };
    try {
        const scriptUrl = new URL(script);
        const pathName = path.join(externalFilePath, scriptUrl.hostname, scriptUrl.pathname);
        if (!fs.existsSync(pathName)) {
            // Axios will throw for any error response
            const response = await axios.get(script);
            fs.mkdirSync(path.dirname(pathName), { recursive: true });
            fs.writeFileSync(pathName, response.data);
        }
        result.localFile = pathName;
    }
    catch (err) {
        result.error = err.message;
    }
    return result;
};

module.exports.saveExternalScript = saveExternalScript;
module.exports.shouldSaveFile = shouldSaveFile;
