'use strict';

const consoleLogger = require('ci-logger');
const { reporterTypes } = require('../lib/reporter');

const testResultSymbols = Object.freeze({
    passed: ' √',
    failed: ' ×',
    warning: ' ‼'
});

const nullFunction = () => { };

// eslint-disable-next-line max-lines-per-function
module.exports = (config) => {
    const cliReporter = {
        // If cli reporter is enabled, set console log function, otherwise null function
        log: config.reporters.includes(reporterTypes.cli) ? consoleLogger.log : nullFunction,
        // The logAlways function provides a mechanism to log to the console even
        // when the cli reporter is disabled (e.g. returning the final results)
        logAlways: consoleLogger.log
    };

    const testResults = {
        project: config.project,
        executionStart: new Date(),
        summary: {
            tests: 0,
            passed: 0,
            warning: 0,
            failed: 0
        },
        results: []
    };

    let currentUrlTests;
    const startUrlTests = (url) => {
        const urlTestResults = {
            url,
            tests: []
        };
        testResults.results.push(urlTestResults);
        currentUrlTests = urlTestResults;
        cliReporter.log({ message: `\nTesting URL: ${url}` });
    };

    const logTestResults = (testResult) => {
        if (testResult) {
            currentUrlTests.tests.push(testResult);
            testResults.summary.tests++;
            testResults.summary[testResult.result]++;

            cliReporter.log({
                message: `${testResult.name} (${testResult.result})`,
                isResult: true,
                resultPrefix: testResultSymbols[testResult.result]
            });
        }
    };

    const outputTestSummary = () => {
        cliReporter.logAlways({ message: `\n Tests: ${testResults.summary.tests}\n Passed: ${testResults.summary.passed}\n Warning: ${testResults.summary.warning}\n Failed: ${testResults.summary.failed}\n` });
    };

    const getTestResults = () => {
        outputTestSummary();
        return testResults;
    };

    return {
        startUrlTests,
        logTestResults,
        getTestResults
    };
};
