'use strict';

const { red } = require('kleur');

const getDataKey = (instancePath) => {
    const baseKey = '<pageanrc>';
    // Special handling for empty pointer or root element
    if (!instancePath || instancePath === '/') {
        return baseKey;
    }
    // Convert pointer to key be splitting by /, removing first value (since pointer
    // starts with /, so empty), unencoding values, and the re-assembling with different
    // formatting for numeric array indices versus properties.
    return baseKey + instancePath.split('/').slice(1).reduce((accumulator, currentValue) => {
        const unencodedValue = currentValue.replace(/~0/g, '~').replace(/~1/g, '/');
        // eslint-disable-next-line sonarjs/no-nested-template-literals -- less intuitive
        return `${accumulator}${isNaN(unencodedValue) ? `.${unencodedValue}` : `[${unencodedValue}]`}`;
    }, '');
};

const processErrorParams = (error) => {
    let formattedMessage = error.message;
    // Ajv includes all allowed values in params.allowedValues,
    // so add to message if available.
    if (error.params.allowedValues) {
        formattedMessage += ` (${error.params.allowedValues.join(', ')})`;
    }
    return formattedMessage;
};

const formatErrors = (errors) => {
    const margin = 2;
    let maxLength = 0;
    errors.forEach(error => {
        error.dataKey = getDataKey(error.instancePath);
        error.formattedMessage = processErrorParams(error);
        // Get max dataKey length for all errors to line up column in final output
        if (error.dataKey.length > maxLength) {
            maxLength = error.dataKey.length;
        }
    });
    return errors.map(error => `  ${error.dataKey.padEnd(maxLength + margin)}${red(error.formattedMessage)}`);
};

module.exports.formatErrors = formatErrors;
