'use strict';

const testResultStates = Object.freeze({
    passed: 'passed',
    warning: 'warning',
    failed: 'failed'
});

const getTestSettings = (testSettingProperty, urlSettings) => {
    if (urlSettings[testSettingProperty] === undefined) {
        throw new Error(`Property '${testSettingProperty}' could not be found in test settings`);
    }
    return urlSettings[testSettingProperty];
};

// Allow cognitive complexity les than 10
// eslint-disable-next-line sonarjs/cognitive-complexity
const pageanTest = async (name, testFunction, testContext, testSettingProperty) => {
    const testSettings = getTestSettings(testSettingProperty, testContext.urlSettings);
    testContext.testSettings = testSettings;
    if (testSettings.enabled) {
        let results;
        try {
            results = await testFunction(testContext);
        }
        catch (err) {
            results = { result: testResultStates.failed, data: { error: err } };
        }
        if (results.result === testResultStates.failed && testSettings.failWarn) {
            results.result = testResultStates.warning;
        }
        testContext.logger.logTestResults({
            name,
            ...results
        });
    }
};

module.exports.testResultStates = testResultStates;
module.exports.pageanTest = pageanTest;
